/* main.h
 *       Copyright (C) 2002 Timo Schulz
 *
 * This file is part of OpenCDK.
 *
 * OpenCDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * OpenCDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCDK; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef CDK_MAIN_H
#define CDK_MAIN_H

#include "types.h"
#include "context.h"

#define MAX_MPI_BITS 8192
#define MAX_MPI_BYTES (MAX_MPI_BITS/8)

#define IS_UID_SIG(s) (((s)->sig_class & ~3) == 0x10)
#define IS_UID_REV(s) ((s)->sig_class == 0x30)

#define DEBUG_PKT (_cdk_get_log_level () == (CDK_LOG_DEBUG+1))

#define KEY_CAN_ENCRYPT(a) (_cdk_pk_algo_usage((a)) & GCRY_PK_USAGE_ENCR)
#define KEY_CAN_SIGN(a)    (_cdk_pk_algo_usage((a)) & GCRY_PK_USAGE_SIGN)

#define init_packet(ctx) memset ((ctx), 0, sizeof (ctx))


/*-- main.c --*/
int _cdk_get_log_level (void);
void _cdk_log_info (const char * fmt, ...);
void _cdk_log_debug (const char * fmt, ...);
char * _cdk_passphrase_get (const char * prompt);
void _cdk_passphrase_free (char * pw);

/*-- misc.c --*/
u32 make_timestamp (void);
u32 buffer_to_u32 (const byte * buf);
void u32_to_buffer (u32 u, byte * buf);
const char* ascii_memistr (const char * buf, size_t buflen, const char * sub);

/*-- pubkey.c --*/
int _cdk_pk_algo_usage (int algo);
int _cdk_pk_algo_usage (int algo);
int _cdk_pk_test_algo (int algo, unsigned int usage_flags);

/*-- cipher.c --*/
int _cdk_cipher_test_algo (int algo);
int _cdk_md_test_algo (int algo);

/*-- new-packet.c --*/
byte * _cdk_subpkt_get_array (CDK_SUBPKT s, int count, size_t * r_nbytes);
int _cdk_subpkt_add (CDK_SUBPKT root, CDK_SUBPKT node);
int _cdk_subpkt_copy (struct cdk_subpkt_s ** r_dst, struct cdk_subpkt_s * src);
int _cdk_subpkt_hash (CDK_SUBPKT hashed, size_t * r_nbytes, GCRY_MD_HD hd);

/*-- sig-check.c --*/
void _cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD hd);
int _cdk_signature_check (cdkPKT_public_key * pk, cdkPKT_signature *sig,
                          GCRY_MD_HD digest, int * r_expired);
void _cdk_hash_sig_data (cdkPKT_signature * sig, GCRY_MD_HD hd);

/*-- kbnode.c --*/
void cdk_kbnode_delete (CDK_KBNODE node);
void cdk_kbnode_add (CDK_KBNODE root, CDK_KBNODE node);
void cdk_kbnode_insert (CDK_KBNODE root, CDK_KBNODE node, int pkttype);
CDK_KBNODE cdk_kbnode_find_prev (CDK_KBNODE root, CDK_KBNODE node, int pktype);
CDK_KBNODE cdk_kbnode_find_next (CDK_KBNODE node, int pkttype);
CDK_KBNODE cdk_kbnode_find (CDK_KBNODE node, int pkttype);
CDK_KBNODE cdk_kbnode_walk (CDK_KBNODE root, CDK_KBNODE * context, int all);

#define is_deleted_kbnode(a)  ((a)->private_flag & 1)
#define is_cloned_kbnode(a)   ((a)->private_flag & 2)

/*-- sesskey.c --*/
int _cdk_pkcs1_sesskey (GCRY_MPI * esk, CDK_DEK dek, int nbits);
int _cdk_pkcs1_digest (byte ** r_md, size_t * r_mdlen, int pk_algo,
                       const byte * md, int digest_algo, unsigned nbits);

/*-- seckey.c --*/
int cdk_hash_passphrase (CDK_DEK dek, char * pw, CDK_S2K_HD * s2k,int creat);

/*-- keydb.c --*/
u32 _cdk_keyid_get (CDK_PACKET * pkt, u32 * keyid);
int _cdk_fingerprint_get (CDK_PACKET * pkt, byte * fpr);
     
CDK_KBNODE _cdk_keydb_get_pkblock (u32 * keyid);
CDK_KBNODE _cdk_keydb_get_skblock (u32 * keyid);
CDK_KBNODE _cdk_keydb_find_bykeyid (CDK_KBNODE root, u32 * keyid);

/*-- sign.c --*/
int _cdk_sig_create (cdkPKT_public_key * pk, cdkPKT_signature *sig);
int _cdk_sig_hash_for (int pubkey_algo, int pkt_version);
void _cdk_trim_string (char * s, int canon);

/*-- stream.c --*/
void * _cdk_stream_get_opaque (CDK_STREAM s, int id);
const char * _cdk_stream_get_fname (CDK_STREAM s);
int _cdk_stream_gets (CDK_STREAM s, char * buf, size_t count);

/*-- verify.c --*/
void _cdk_result_verify_free (Verify_Result res);
Verify_Result _cdk_result_verify_new (void);

/*-- encrypt.c --*/
int _cdk_proc_signature_packets (CDK_HD hd, GCRY_MD_HD md,
                                 CDK_STREAM inp, CDK_STREAM out);

#endif /* CDK_MAIN_H */
